@@ -1,4 +1,3 @@
-use crate::proto::Datum;
 use crate::Query;
 use serde_json::Value;
 
@@ -6,11 +5,17 @@ pub trait Arg {
     fn into_query(self) -> Query;
 }
 
+impl Arg for Query {
+    fn into_query(self) -> Query {
+        self
+    }
+}
+
 impl<T> Arg for T
 where
     T: Into<Value>,
 {
     fn into_query(self) -> Query {
-        Datum::from(self.into()).into()
+        Query::from_json(self)
     }
 }
